<?php
/*
Plugin Name: WP Simple Social Poll
Plugin URI: http://socpoll.kniga5.ru/?ch=plugin
Description: Плагин позволяет создавать голосования, в которых пользователи голосуют при помощи социальных кнопок Like, Tweet, "Мне нравится".
Version: 0.2
Author: Алексей Жизневский
Author URI: http://prog.kniga5.ru
*/

/*  Copyright 2012  Alexey Zhiznevskiy  (email : write2aracon {at} gmail.com)

  

*/


class WPSimpleSocialPoll 
{
	function WPSimpleSocialPoll() {
		register_activation_hook( __FILE__, array( $this, 'install' ) );
		
		add_shortcode('socialpoll', array($this, 'poll_shortcode'));
		
		add_action( 'wp_enqueue_scripts', array($this, 'enqueue_styles') );
		add_action('wp_ajax_poll_social', array($this, 'poll_social_ajax_callback'));
		add_action('wp_ajax_nopriv_poll_social', array($this, 'poll_social_ajax_callback'));
		add_action( 'save_post', array($this, 'hook_save_post') );
		
		if(is_admin()) {
			add_action('admin_menu', array($this, 'add_admin_pages'));
			add_action('admin_init', array($this, 'register_mysettings') );
		}
	}
	
	function enqueue_styles() {
		//wp_enqueue_script( 'json2' );
		
		wp_enqueue_script('vk_script', 'http://userapi.com/js/api/openapi.js?49');
		
		wp_enqueue_style('socialpoll_styles', plugins_url('style.css', __FILE__ ));
		wp_enqueue_script('socialpoll_ajax_script', plugins_url('socpoll.js', __FILE__ ), array( 'jquery', 'json2', 'vk_script' ));
		
		// declare the URL to the file that handles the AJAX request (wp-admin/admin-ajax.php)
		wp_localize_script( 'socialpoll_ajax_script', 'MyAjax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );
	}
	
	function add_admin_pages() {
		add_options_page('Настройки социального голосования', 'Социальное голосование', 'manage_options', __FILE__ , array($this, 'admin_page'));
	}
	
	function admin_page() {
		//echo "<h2>Options</h2>";
		include dirname(__FILE__).'/options.page.php';
	}
	
	function register_mysettings() {
		register_setting( 'simplesocialpoll-settings', 'vk_site_id', 'intval' );
		register_setting( 'simplesocialpoll-settings', 'socpoll_vk_repost_link');
		register_setting( 'simplesocialpoll-settings', 'socpoll_vk_repost_text');
		register_setting( 'simplesocialpoll-settings', 'socpoll_vk_repost_image_link');
		register_setting( 'simplesocialpoll-settings', 'socpoll_vk_repost_page_descr');
	}
	
	function poll_shortcode($atts, $content = null) {
		global $post;
		
		$poll = $atts = shortcode_atts(
			array(
				'id' => 0,
				'title' => '',
			),
			$atts
		);
		

		
		$poll['post_id'] = $post_id = intval($post->ID);
		$poll['poll_id'] = $poll_id = intval($poll['id']);
		
		
		$variants = $this->extract_variants($content);
		$poll['variants'] = $variants;
		
		
		//Debug_Bar_Extender::instance()->trace_var( $variants );
		
		
		$votes = $this->get_votes($post_id, $poll_id);
		//Debug_Bar_Extender::instance()->trace_var( $votes );
		
		$this->add_new_variants($poll, $votes);
		$this->update_variants_titles($poll);
		$poll['variants'] = $this->merge_variants_and_votes($poll['variants'], $votes);
		$poll['variants'] = $this->update_votes_percent($poll['variants']);
		
		
		return $this->theme_poll($poll);
	}
	
	function hook_save_post($post_id) {
		global $wpdb;
		$table_name = $wpdb->prefix . "soc_poll_votes";
		
		$wpdb->query("UPDATE ".$table_name." SET variant_name = NULL, poll_title = NULL WHERE post_id = ".intval($post_id));
	}
	
	function merge_variants_and_votes($variants, $votes) {
		for($i=0; $i < count($variants); $i++) {
			$id = $variants[$i]['id'];
			if(array_key_exists($id, $votes)) {
				$variants[$i]['votes'] = $votes[$id];
			} else {
				$variants[$i]['votes'] = 0;
			}
		}
		return $variants;
	}
	
	function update_votes_percent($variants) {
		$votes_total = 0;
		for($i=0; $i < count($variants); $i++) {
			$votes_total += $variants[$i]['votes'];
			$variants[$i]['votes_percent'] = 0;
		}
		if($votes_total != 0) {
			for($i=0; $i < count($variants); $i++) {
				$variants[$i]['votes_percent'] = round( 100*$variants[$i]['votes'] / $votes_total);
			}
		}
		return $variants;
	}
	
	function get_votes($post_id, $poll_id) {
		global $wpdb;
		$table_name = $wpdb->prefix . "soc_poll_votes";
		
		$post_id = intval($post_id);
		$poll_id = intval($poll_id);
		
		$votes_rows = $wpdb->get_results("SELECT variant_id, votes FROM ".$table_name." WHERE post_id = ".$post_id." AND poll_id = ".$poll_id." ORDER BY variant_id ASC");
		
		$votes = array();
		foreach($votes_rows as $row) {
			$votes[intval($row->variant_id)] = intval($row->votes);
		}
		
		return $votes;
	}
	
	function convert_votes_to_variants($votes) {
		$variants = array();
		foreach($votes as $id => $cnt) {
			$variants[] = array(
				'id' => $id,
				'votes' => $cnt,
			);
		}
		return $variants;
	}
	
	
	
	function add_new_variants($poll, $votes_in_db) {
		$variants = $poll['variants'];
		$post_id = intval($poll['post_id']);
		$poll_id = intval($poll['poll_id']);
		
		$newlist = array();
		
		foreach($variants as $v) {
			if(!array_key_exists($v['id'], $votes_in_db)) {
				$newlist[] = array('id' => intval($v['id']), 'title' => $v['title']);
				
			}
		}
		
		if(count($newlist)==0) return;
		
		global $wpdb;
		$table_name = $wpdb->prefix . "soc_poll_votes";
		
		$sql = "INSERT IGNORE INTO ".$table_name." (post_id, poll_id, variant_id, votes, variant_name, poll_title) VALUES";
		$first = TRUE;
		foreach($newlist as $add) {
			if(!$first) {
				$sql .= ",";
			}
			$first = FALSE;
			$sql .= $wpdb->prepare("(%d,%d,%d,0,%s,%s)", $post_id, $poll_id, $add['id'], $add['title'], $poll['title']);
		}
		
		$wpdb->query($sql);
	}
	
	function update_variants_titles($poll) {
		global $wpdb;
		$table_name = $wpdb->prefix . "soc_poll_votes";
		
		$toupdate = $wpdb->get_results("SELECT variant_id FROM ".$table_name." WHERE post_id = ".intval($poll['post_id'])." AND poll_id = ".intval($poll['poll_id'])." AND variant_name IS NULL");
		
		$ids = array();
		foreach($toupdate as $row) {
			$ids[] = $row->variant_id;
		}
		
		foreach($poll['variants'] as $v) {
			if(in_array($v['id'], $ids)) {
				$wpdb->update($table_name, array('variant_name' => $v['title'], 'poll_title' => $poll['title']), array('post_id' => $poll['post_id'], 'poll_id' => $poll['poll_id'], 'variant_id' => $v['id']),
											array('%s', '%s'), array('%d', '%d', '%d'));
			}
		}
		
	}
	
	
	function extract_variants($content) {
		$content = str_replace("<p>", "", $content);
		$content = str_replace("</p>", "", $content);
		$content = str_replace("<br>", "", $content);
		$content = str_replace("<br/>", "", $content);
		$content = str_replace("<br />", "", $content);
		
		$variants = array();
		
		$lines = explode("\n", $content);
		$i=0;
		foreach($lines as $line) {
			$line = str_replace("\r", '', $line);
			$line = trim($line);
			
			if($line != "") {
				$line_x = explode('|', $line);
				if(count($line_x)==2) {
					$variant_id = intval($line_x[0]);
					$variants[] = array('id' => $variant_id, 'title' => $line_x[1]);
				} else {
					$variants[] = array('id' => $i, 'title' => $line);
				}
				$i++;
			}
		}
		
		
		
		return $variants;
	}
	
	
	function poll_social_ajax_callback() {
		$post_id = intval($_POST['postid']);
		$poll_id = intval($_POST['pollid']);
		$variant_id = intval($_POST['variant']);
		
		$cookie_name = 'socpoll_voted_'.$post_id.'-'.$poll_id;
		
		$votes = $this->get_votes($post_id, $poll_id);
		
		if(!isset($_COOKIE[$cookie_name]) || $_COOKIE[$cookie_name]!='yes') {
			$votes[$variant_id]++;
			
			global $wpdb;
			$table_name = $wpdb->prefix . "soc_poll_votes";
			$wpdb->query($wpdb->prepare("UPDATE ".$table_name." SET votes=votes+1
					WHERE post_id = %d AND poll_id = %d AND variant_id = %d",
					$post_id, $poll_id, $variant_id));
		}
			
		
		$variants = $this->convert_votes_to_variants($votes);
		$variants = $this->update_votes_percent($variants);
		
		$newVotes = array();
		foreach($variants as $v) {
			$newVotes[$v['id']]['votes'] = $v['votes'];
			$newVotes[$v['id']]['votes_percent'] = $v['votes_percent'];
		}
		
		$data = array(
			'variants' => count($votes),
			'newVotes' => $newVotes,
		);
		
		// generate the response
    $response = json_encode( array( 'success' => true, 'voteData' => $data ) );
 
    // response output
    //header( "Content-Type: application/json" );
    setcookie($cookie_name, 'yes', time()+60*60*24*30*12);
    echo $response;
 
    // IMPORTANT: don't forget to "exit"
    exit;
	}
	
	
	
	function install() {
		global $wpdb;
		
		$table_name = $wpdb->prefix . "soc_poll_votes";
		
		$sql = "CREATE TABLE ".$table_name." (
			post_id bigint(20) UNSIGNED NOT NULL,
			poll_id SMALLINT UNSIGNED NOT NULL,
			variant_id SMALLINT UNSIGNED NOT NULL,
			votes BIGINT UNSIGNED NOT NULL,
			variant_name TINYTEXT,
			poll_title TINYTEXT,
			PRIMARY KEY (post_id, poll_id, variant_id)
		);";
		
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);
		
		add_option('socpoll_secret_key', rand() );
	}
	
	function theme_poll($poll) {
		$ret = "<h3>".$poll['title']."</h3>";
		$ret .= '<div class="pollBarsEnv" id="socpoll-'.$poll['post_id'].'-'.$poll['poll_id'].'">';
		$ret .= $this->theme_poll_bars($poll);
		$ret .= '</div>';
		return $ret;
	}
	
	function theme_poll_bars($poll) {
		$site_id = get_option('vk_site_id');;
		//$salt = get_option('socpoll_secret_key', '0');
		
		$repost_link = get_option('socpoll_vk_repost_link', get_site_url());
		$repost_text = get_option('socpoll_vk_repost_text', wp_title('|', false));
		$repost_image = get_option('socpoll_vk_repost_image_link', '');
		$repost_descr = get_option('socpoll_vk_repost_page_descr', wp_title('|', false));
		
		//$ret = "<div class='pollBars'>";
		$ret = "<table class='pollBars'>";
		$pollfid = $poll['post_id'].'-'.$poll['poll_id'];
		foreach($poll['variants'] as $v) {
			$vfid = $pollfid.'-'.$v['id'];
			//$ret .= "<br/><b>".$v['title']."</b> ";
			$ret .= '<tr><td>';
			$ret .= "<strong>".$v['title']."</strong> ";
			$span_id = 'socpollbar-'.$vfid;
			$ret .= '<span class="t1" id="'.$span_id.'" style="width: '.$v['votes_percent'].'%">'.$v['votes'].'</span>';
			//$ret .= "<input type='button' value='Голосовать!' onClick='WPSocialPollVote(".$poll['post_id'].",".$poll['poll_id'].",".$v['id'].")' />";
			/*$ret .= '<div id="vk_like_poll_'.$vfid.'"></div>
<script type="text/javascript">
//VK.Widgets.Like("vk_like_poll_'.$vfid.'", {type: "mini"});
</script>';*/

			//$control = md5($salt.$repost_link.$repost_text);

			$ret .= '</td><td width="220">';
			$ret .= '<iframe class="socButtons" src="'.plugins_url('likeframe.php', __FILE__ ).'?post='.$poll['post_id'].'&poll='.$poll['poll_id'].'&variant='.$v['id'].'&site='.$site_id.'"
			width="210" height="165" scrolling="no" allowtransparency="true"></iframe>';
			$ret .= '</td>';
			$ret .= '</tr>';
		}
		//$ret .= '</div>';
		$ret .= '</table>';
		$ret .= '<span style="font-size: 9px; margin-top: 0; margin-bottom: 7px;"><a href="http://socpoll.kniga5.ru/?ch=plugin">Плагин социального голосования</a></span><br/>';
		return $ret;
	}
}


add_action('activated_plugin','save_error');
function save_error(){
    update_option('plugin_error',  ob_get_contents());
}

$wpsimplesocialpoll = new WPSimpleSocialPoll();


